# fungus garden analysis script
#
# Kevin Lee, Klassen Lab, MCB, UConn, Storrs, CT
#
# Set the working directory (This is individual to the machine running the code)

# Load the required packages

library(dada2)
library(phyloseq)
library(ggplot2)
library(plyr)
library(dplyr)
library(decontam)

# Set path to fastq files -- This needs to be customized to your workflow

path <- file.path(".")

# Read in file names, the pattern portion of this command needs to be consistent with the naming of your fastq files

fnFs <- sort(list.files(path, pattern = "_R1.fastq", full.names = TRUE))
fnRs <- sort(list.files(path, pattern = "_R2.fastq", full.names = TRUE))

# Assign sample names, as constructed, this command will give your samples names according to everything before the first use of underscore, if you have a different character you want to use for pattern matching, change this command accordingly

sample.names <- sapply(strsplit(basename(fnFs), "_"), '[', 1)

# Sanity check step, you can generate a composite of the quality profiles of your reads, similar to a fastQC report. This can be helpful later if you have results that don't look like what you expect. The numbers in brackets should be the number of samples you have.

f_qual <- plotQualityProfile(fnFs[1:48], aggregate = TRUE)
r_qual <- plotQualityProfile(fnRs[1:48], aggregate = TRUE)

jpeg(filename = "forward_quality_profile.jpg", width = 800, height = 600)
plot(f_qual)
dev.off()

jpeg(filename = "reverse_quality_profile.jpg", width = 800, height = 600)
plot(r_qual)
dev.off()

# Filtering and Trimming

# Setting paths for filtered files to be saved

filt_path <- file.path(path, "filtered")
filtFs <- file.path(filt_path, paste0(sample.names, "_F_filt.fastq.gz"))
filtRs <- file.path(filt_path, paste0(sample.names, "_R_filt.fastq.gz"))

# Filtering the reads: the truncLen parameter should be decided by looking at the quality profile of your reads. This can be done prior to running the script using the commands above. The maxEE parameter filters by expected error rates therefore higher numbers are less stringent than lower numbers. Multithread can be set to TRUE if being run in native R in a non-Windows environment (Linux or MacOS).

out <- filterAndTrim(fnFs, filtFs, fnRs, filtRs, truncLen = c(240, 200), maxN = 0, maxEE = c(2,2), truncQ = 2, rm.phix = TRUE, compress = TRUE, multithread = FALSE)

# Learning error profiles

errF <- learnErrors(filtFs, multithread = FALSE)
errR <- learnErrors(filtRs, multithread = FALSE)

# Sanity check: Plot error profiles

errF_plot <- plotErrors(errF, nominalQ = TRUE)
errR_plot <- plotErrors(errR, nominalQ = TRUE)

jpeg(filename = "F_error.jpg", width = 800, height = 600)
plot(errF_plot)
dev.off()

jpeg(filename = "R_error.jpg", width = 800, height = 600)
plot(errR_plot)
dev.off()

# Dereplicating samples for computational efficiency

derepFs <- derepFastq(filtFs, verbose = TRUE)
derepRs <- derepFastq(filtRs, verbose = TRUE)

names(derepFs) <- sample.names
names(derepRs) <- sample.names

# Inferring sequence variants

dadaFs <- dada(derepFs, err = errF, multithread = FALSE)
dadaRs <- dada(derepRs, err = errR, multithread = FALSE)

# Merging read pairs

mergers <- mergePairs(dadaFs, derepFs, dadaRs, derepRs, verbose = TRUE)

# Make the sequence table. The output of the merger may assemblies that are way outside the range of V4 length, I recommend making a second sequence table that has the length range "cut-out". That is the function of the seqtab2 command below. 

seqtab <- makeSequenceTable(mergers)

# Removing Chimeras, including a sanity check step that prints the output of these steps to a pdf

seqtab.nochim <- removeBimeraDenovo(seqtab, method = "consensus", multithread = FALSE, verbose = TRUE)

seqtab_size <- dim(seqtab)
no.chim_size <- dim(seqtab.nochim)
kept <- sum(seqtab.nochim)/sum(seqtab)
cat("Dimensions of seqtab", seqtab2_size, "Dimensions of seqtab.nochim", no.chim_size, "Percent of reads remaining after chimera removal", kept, file = "chimera_report.txt", fill = FALSE, sep = "\n")

# Tracking reads through the system, the last sanity check. Makes a csv that shows the read counts of each sample after each step of the process

getN <- function(x) sum(getUniques(x))
track <- cbind(out, sapply(dadaFs, getN), sapply(mergers, getN), rowSums(seqtab2), rowSums(seqtab.nochim))
colnames(track) <- c("Input", "Filtered", "Denoised", "Merged", "Tabled", "Non-Chimeric")
write.csv(track, file = "Read_fates.csv", row.names = TRUE)

# Assigning taxonomy. You need to have the reference database file downloaded and make sure the path in the command agrees

taxa <- assignTaxonomy(seqtab.nochim, "silva_nr_v128_train_set.fa.gz", multithread = FALSE)

# Making a tree. Useful if you are going to do phylogeny based analyses downstream. Also handy as part of a complete phyloseq object, the added computational time may be worthwhile as opposed to backtracking and trying to do this as a separate step later. Requires DEICPHER and phangorn.

library(DECIPHER)
library(phangorn)

seqs <- getSequences(seqtab)
names(seqs) <- seqs
alignment <- AlignSeqs(DNAStringSet(seqs), anchor = NA)

phang.align <- phyDat(as(alignment, "matrix"), type = "DNA")
dm <- dist.ml(phang.align)
treeNJ <- NJ(dm)
fit <- pml(treeNJ, data = phang.align)
fitGTR <- update(fit, k=4, inv = 0.2)
fitGTR <- optim.pml(fitGTR, model = "GTR", optInv = TRUE, optGamma = TRUE, rearrangement = "stochastic", control = pml.control(trace =0))
detach("package:phangorn", unload = TRUE)

# Building the phyloseq object. Any metadata needs to be read in before, as shown in the meta command.

meta <- read.csv("dada2_meta.csv", header = TRUE, row.names = 1)

ps0 <- phyloseq(tax_table(taxa), otu_table(seqtab.nochim, taxa_are_rows = FALSE), sample_data(meta), phy_tree(fitGTR$tree))

# Removing unwanted elements

ps1 <- subset_taxa(ps0, !is.na(Phylum))

ps2 <- subset_taxa(ps1, !Class == "Chloroplast")

# Running decontam
# Removing negative control 1 sample with no reads (causes error if left in)

ps3 <- subset_samples(ps2, !sample_names(ps2) == "nc1")

sample_data(ps3)$is.neg <- sample_data(ps3)$type == "negative"

contamdf.prev <- is.Contaminant(ps3, method = "Prevalence", threshold = 0.5, neg = "is.neg"

# Remove contaminants

contaminants <- rowname(contamdf.prev)[(contamdf.prev$contaminant == "TRUE")]

ps4 <- subset_taxa(ps3, !taxa_names(ps3) %in% contaminants)

#Removing controls and samples not used in this study

neg <- sample_names(ps4)[(sample_data(ps4)$type == "negative")]
lab <- sample_names(ps4)[(sample_data(ps4)$stata == "LAB")]

neg_lab <- paste(c(neg,lab))

ps5 <- subset_samples(ps4, !sample_names(ps4) %in% neg_lab)

ps6 <- subset_samples(ps5, sample_sums(ps5) > 7000)

set.seed(100)

ps7 <- rarefy_even_depth(ps6, rngseed = TRUE)

ps8 <- subset_taxa(ps7, !Family == "Mitochondria")

singletons <- c("jka1226", "jka1249", "jka9712")

ps9 <- subset_samples(ps8, !sample_names(ps8) %in% singletons)

ps9rab <- transform_sample_counts(ps9, function(x) {x/sum(x)})

#Phylum level barplot

ps9rabglom <- tax_glom(ps9rab, taxrank = "Phylum")

ps9rabglomdat <- psmelt(ps9rabglom)

ps9rabglomdat$Phylum <- as.character(ps9rabglomdat$Phylum)

ps9rabglomdat$preservative <- as.character(ps9rabglomdat$preservative)

maximum <- ddply(ps9rabglomdat, ~Phylum, function(x) c(maximum = max(x$Abundance)))
Other <- maximum[maximum$maximum <= 0.15,]$Phylum

ps9rabglomdat[ps9rabglomdat$Phylum %in% Other,]$Phylum <- "Other"

ps9rabglomdat[ps9rabglomdat$Sample == "jka535",]$preservative <- "GLY2"

ps9rabglomdat[ps9rabglomdat$Sample == "jka668h",]$preservative <- "PBS2"

ps9rabglomdat[ps9rabglomdat$Sample == "jka1276h",]$preservative <- "GLY2"

phylum_bar_plot <- ggplot(ps9rabglomdat, aes(x = preservative, y = Abundance, fill = Phylum)) + geom_bar(stat = "identity") + facet_grid(~colony, scales = "free", space = "free") + scale_fill_manual(values = c("Acidobacteria" = "#CC9983", "Actinobacteria" = "Yellow", "Firmicutes" = "Sky Blue", "Planctomycetes" = "#009980", "Proteobacteria" = "Orange", "Other" = "Black"), breaks = c("Acidobacteria", "Actinobacteria", "Firmicutes", "Planctomycetes", "Proteobacteria", "Other")) + scale_y_continuous(expand = c(0,0)) + theme(panel.background = element_blank(), strip.background = element_blank(), strip.text = element_text(hjust = 0.5, vjust = 0.5, face = "bold"), axis.text.x = element_text(angle = 270, hjust = 0, vjust = 0.5, face = "bold", size = 10), axis.text.y = element_text(face = "bold", size = 10), axis.title.y = element_text(face = "bold", size = 14), axis.title.x = element_blank(), legend.title = element_text(face = "bold", size = 12), legend.text = element_text(face = "bold", size = 10))

# PCoA of Weighted Unifrac

ps9rab.wuf.dist <- distance(ps9rab, method = "wunifrac")

ps9rab.wuf.ord <- ordinate(ps9rab, method = "PCoA", distance = "wunifrac")

ps9rab.wuf.plot <- plot_ordination(ps9rab, ps9rab.wuf.ord, color = "preservative") + labs(col = "Preservative") + geom_point(size = 4) + theme(panel.background = element_blank(), panel.border = element_rect(fill = NA), axis.title = element_text(size = 12, face = "bold"), axis.text = element_text(size = 10, face = "bold"), legend.key = element_blank(), legend.text = element_text(face = "bold", size = 10), legend.title = element_text(face = "bold", size = 12))

# PERMANOVA of fungus garden samples

ps9data <- sample_data(ps9rab)
ps9data <- data.frame(ps9data)
adonis(ps9rab.wuf.dist ~ colony, ps9data)
adonis(ps9rab.wuf.dist ~ preservative, ps9data)
adonis(ps9rab.wuf.dist ~ colony * preservative, ps9data)

# Alpha diversity

ps9rich_est <- estimate_richness(ps9, measures = c("Shannon", "Chao1", "Observed"))

ps9data$rows <- rownames(ps9data)

ps9rich_est$rows <- rownames(ps9rich_est)

ps9_diversity <- left_join(ps9data, ps9rich_est)

shannon_aov <- aov(Shannon ~ preservative, ps9_diversity)

chao1_aov <- aov(Chao1 ~ preservative, ps9_diversity)

observed_aov <- aov(Observed ~ preservative, ps9_diversity)

ps9_div_plot <-  plot_richness(ps9, x = "preservative", measures = c("Shannon", "Chao1", "Observed")) + geom_boxplot(aes(fill = preservative), show.legend = FALSE, width = 0.5) + theme(panel.background = element_blank(), panel.border = element_rect(fill = NA), strip.background = element_blank(), strip.text = element_text(face = "bold", size = 12), axis.title.y = element_text(face = "bold", size = 14), axis.text = element_text(face = "bold", size = 10), axis.title.x = element_blank())
